from arrow import get, utcnow
from requests import Session
from re import sub
from json import loads
from operator import attrgetter
from ..response import Response
from .exceptions import ResponseCodeException, ResponseDataException


class Scraper(object):

    def __init__(self, session=None):
        if session and isinstance(session, Session):
            self.session = session
        else:
            self.session = Session()

    def get(self, zone, timezone, system="Canarias", date=None):
        if not date:
            date = self._searchdate(timezone)
        url = self._makeurl(zone, date, system)
        responses = []
        for value in self._request(url):
            ts = value['ts']
            arrow = self._timestamp(ts, timezone)
            response = Response(arrow.float_timestamp)
            response.demand = value['dem']
            response.diesel = value['die']
            response.gas = value['gas']
            response.wind = value['eol']
            response.combined = value['cc']
            if 'vap' in value:
                response.vapor = value['vap']
            response.solar = value['fot']
            if 'hid' in value:
                hidro = value['hid']
                if hidro >= 0.0:
                    response.hydraulic = hidro
                else:
                    response.storage = abs(hidro)
            if 'car' in value:
                response.carbon = value['car']
            if 'cb' in value:
                response.link['pe_ma'] = value['cb']
            responses.append(response)
        return max(responses, key=attrgetter('timestamp'))

    def _request(self, url):
        response = self.session.request("GET", url)
        if response.status_code != 200:
            raise ResponseCodeException
        if not response.text:
            raise ResponseDataException
        return self.__getjson(response.text)

    def _makeurl(self, zone, date, system="Canarias"):
        base = "https://demanda.ree.es/WSvisionaMoviles{2}Rest/resources/demandaGeneracion{2}?curva={0}&fecha={1}"
        return base.format(zone, date, system)

    def __getjson(self, text):
        removed_null = sub("null\(", "", text)
        response_cleaned = sub("\);", "", removed_null)
        json = loads(response_cleaned)
        return json['valoresHorariosGeneracion']

    def _timestamp(self, date_str, timezone):
        return get(date_str + ' ' + timezone, 'YYYY-MM-DD HH:mm ZZZ')

    def _searchdate(self, timezone, arrow=None):
        if not arrow:
            arrow = utcnow()
        return arrow.to(timezone).format('YYYY-MM-DD')






from reescraper.core import Scraper


class Lanzarote(Scraper):

    def __init__(self, session=None):
        super(self.__class__, self).__init__(session)

    def get(self):
        return super(self.__class__, self).get("LANZAROT", "Atlantic/Canary")



from .canary import ElHierro, GranCanaria, Gomera, LanzaroteFuerteventura, LaPalma, Tenerife
from .core import Scraper, NoDataException, TimestampException
from .response import Response


class CanaryIslands(Scraper):

    def __init__(self, session=None):
        super(self.__class__, self).__init__(session)
        self.__responses = []

    def get(self):
        self.__responses = []
        self.__gomera()
        self.__grancanaria()
        self.__hierro()
        self.__lanzarotefuerteventura()
        self.__palma()
        self.__tenerife()
        self.__checkgenerations()
        return self.__response()

    def __response(self):
        timestamp = self.__timestamp()
        response = Response(timestamp)
        response.demand = self.__demand()
        response.diesel = self.__diesel()
        response.gas = self.__gas()
        response.wind = self.__wind()
        response.combined = self.__combined()
        response.vapor = self.__vapor()
        response.solar = self.__solar()
        response.hydraulic = self.__hydraulic()
        response.storage = self.__storage()
        return response

    def __demand(self):
        demand = 0.0
        for gen in self.__responses:
            demand += gen._demand
        return demand

    def __storage(self):
        storage = 0.0
        for gen in self.__responses:
            storage += gen._storage
        return storage

    def __diesel(self):
        diesel = 0.0
        for gen in self.__responses:
            diesel += gen._diesel
        return diesel

    def __gas(self):
        gas = 0.0
        for gen in self.__responses:
            gas += gen._gas
        return gas

    def __wind(self):
        wind = 0.0
        for gen in self.__responses:
            wind += gen._wind
        return wind

    def __combined(self):
        combined = 0.0
        for gen in self.__responses:
            combined += gen._combined
        return combined

    def __vapor(self):
        vapor = 0.0
        for gen in self.__responses:
            vapor += gen._vapor
        return vapor

    def __solar(self):
        solar = 0.0
        for gen in self.__responses:
            solar += gen._solar
        return solar

    def __hydraulic(self):
        hydraulic = 0.0
        for gen in self.__responses:
            hydraulic += gen._hydraulic
        return hydraulic

    def __timestamp(self):
        generation = self.__responses[0]
        timestamp = generation.timestamp
        if not timestamp:
            raise TimestampException
        else:
            return timestamp

    def __checkgenerations(self):
        timestamp = self.__timestamp()
        for gen in self.__responses:
            if gen.timestamp != timestamp:
                raise TimestampException

    def __gomera(self):
        gomera = Gomera(self.session).get()
        if not gomera:
            raise NoDataException
        else:
            self.__responses.append(gomera)

    def __grancanaria(self):
        granacanaria = GranCanaria(self.session).get()
        if not granacanaria:
            raise NoDataException
        else:
            self.__responses.append(granacanaria)

    def __hierro(self):
        hierro = ElHierro(self.session).get()
        if not hierro:
            raise NoDataException
        else:
            self.__responses.append(hierro)

    def __lanzarotefuerteventura(self):
        lanzarotefuerteventura = LanzaroteFuerteventura(self.session).get()
        if not lanzarotefuerteventura:
            raise NoDataException
        else:
            self.__responses.append(lanzarotefuerteventura)

    def __palma(self):
        palma = LaPalma(self.session).get()
        if not palma:
            raise NoDataException
        else:
            self.__responses.append(palma)

    def __tenerife(self):
        tenerife = Tenerife(self.session).get()
        if not tenerife:
            raise NoDataException
        else:
            self.__responses.append(tenerife)



from arrow import get


class Response(object):

    def __init__(self, timestamp, demand=0.0, diesel=0.0, gas=0.0, wind=0.0, combined=0.0, vapor=0.0, solar=0.0, hydraulic=0.0, carbon=0.0):
        self.timestamp = timestamp
        self._demand = demand
        self._diesel = diesel
        self._gas = gas
        self._wind = wind
        self._combined = combined
        self._vapor = vapor
        self._solar = solar
        self._hydraulic = hydraulic
        self._carbon = carbon
        self.link = {}
        self._storage = 0.0

    @property
    def demand(self):
        return round(self._demand, 2)

    @demand.setter
    def demand(self, demand):
        self._demand = demand

    @property
    def diesel(self):
        return round(self._diesel, 2)

    @diesel.setter
    def diesel(self, diesel):
        self._diesel = diesel

    @property
    def gas(self):
        return round(self._gas, 2)

    @gas.setter
    def gas(self, gas):
        self._gas = gas

    @property
    def wind(self):
        return round(self._wind, 2)

    @wind.setter
    def wind(self, wind):
        self._wind = wind

    @property
    def combined(self):
        return round(self._combined, 2)

    @combined.setter
    def combined(self, combined):
        self._combined = combined

    @property
    def vapor(self):
        return round(self._vapor, 2)

    @vapor.setter
    def vapor(self, vapor):
        self._vapor = vapor

    @property
    def solar(self):
        return round(self._solar, 2)

    @solar.setter
    def solar(self, solar):
        self._solar = solar

    @property
    def hydraulic(self):
        return round(self._hydraulic, 2)

    @hydraulic.setter
    def hydraulic(self, hydraulic):
        self._hydraulic = hydraulic

    @property
    def carbon(self):
        return round(self._carbon, 2)

    @carbon.setter
    def carbon(self, carbon):
        self._carbon = carbon

    @property
    def storage(self):
        return round(self._storage, 2)

    @storage.setter
    def storage(self, storage):
        self._storage = storage

    def __str__(self):
        base = "Response {0} Demand {1} Diesel: {2} Gas: {3} Wind: {4} Combined: {5} Vapor: {6} Solar: {7} Hydraulic: {8} Carbon: {9}"
        return base.format(get(self.timestamp),
                           self.demand,
                           self.diesel,
                           self.gas,
                           self.wind,
                           self.combined,
                           self.vapor,
                           self.solar,
                           self.hydraulic,
                           self.carbon)

    def _production(self):
        """Calculate total energy production. Not rounded"""
        return self._diesel + self._gas + self._wind + self._combined + self._vapor + self._solar + self._hydraulic + self._carbon - self._storage

    def production(self):
        """Calculate total energy production."""
        return round(self._production(), 2)

    def _links(self):
        """Calculate total energy production. Not Rounded"""
        total = 0.0
        for value in self.link.values():
            total += value
        return total

    def links(self):
        """Calculate total energy production."""
        return round(self._links(), 2)

    def _unknown(self):
        """Calculate unknown energy production. Not Rounded"""
        return self._demand - (self._production() + self._links())

    def unknown(self):
        """Calculate unknown energy production."""
        return max(0.0, round(self._unknown(), 2))



from .canary import ElHierro, Fuerteventura, GranCanaria, Gomera, Lanzarote, LaPalma, Tenerife, LanzaroteFuerteventura
from .canaryislands import CanaryIslands
from .core.exceptions import NoDataException, TimestampException
from .response import Response
from .balearic import Mallorca, Menorca
from .balearicislands import BalearicIslands



from .lanzarote import Lanzarote
from .elhierro import ElHierro
from .fuerteventura import Fuerteventura
from .gomera import Gomera
from .grancanaria import GranCanaria
from .lapalma import LaPalma
from .tenerife import Tenerife
from .lanzafuerte import LanzaroteFuerteventura



from ..core import Scraper


class Menorca(Scraper):

    def __init__(self, session=None):
        super(self.__class__, self).__init__(session)

    def get(self):
        return super(self.__class__, self).get("MENORCA", "Europe/Madrid", "Baleares")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name='reescraper',
    version='1.3.0',
    packages=['reescraper', 'reescraper/core', 'reescraper/response', 'reescraper/canary', 'reescraper/balearic'],
    install_requires=[
        'arrow',
        'requests'
      ],
    url='https://github.com/blackleg/reescraper',
    license='MIT',
    author='blackleg',
    author_email='hectorespertpardo@gmail.com',
    description='Red Eléctrica de España data scraper'
)
